'use strict'

require('./assets/css/main.css')

const TweenMax = require('gsap')
const ipcRenderer = require('electron').ipcRenderer
const weatherLib = new (require('vixonic-weather-360'))

var initialized = false
var temperatureView, iconContainerView, lastUpdateView, cityView, backgroundView

var weatherData

window.onload = () => {
  console.log('Module loaded!')

  temperatureView = document.getElementById('temperature')
  iconContainerView = document.getElementById('icon-container')
  lastUpdateView = document.getElementById('last-update')
  backgroundView = document.getElementById('background')
  cityView = document.getElementById('city')
  playAnimation('end', 0)

  weatherLib.on('updated', (data) => {
    if (!initialized) {
      initialized = true
      weatherData = JSON.parse(JSON.stringify(data))
      loadIcon((icon, prevIcons) => {
        icon.style.opacity = 1
        playAnimation('start')
      })
    } else {
      if (!compareAttr('cityName', weatherData, data)) {
        weatherData.cityName = data.cityName
        updateCity(true)
      }
      if (!compareAttr('temperature', weatherData, data)) {
        weatherData.temperature = data.temperature
        updateTemperature(true)
      }
      if (!compareAttr('iconBase', weatherData, data)) {
        weatherData.iconBase = data.iconBase
        updateIcon(true)
      }
      if (!compareAttr('lastUpdate', weatherData, data)) {
        weatherData.lastUpdate = data.lastUpdate
        updateLastUpdate(true)
      }
    }
  })
}

function playAnimation (type, duration = 1 , animationFinished = null) {
  switch (type) {
    case 'start':
      updateCity()
      TweenMax.to(cityView, duration, { y: '0px', alpha: 1 })
      TweenMax.to(iconContainerView, duration, { scale: '1', alpha: 1, delay: 0.5 })
      updateTemperature()
      TweenMax.to(temperatureView, duration, { scale: '1', alpha: 1, delay: 1 })
      updateLastUpdate()
      TweenMax.to(lastUpdateView, duration, { y: '0px', alpha: 1 })
      break
    case 'end':
      TweenMax.to(cityView, duration, { y: '-50px', alpha: 0 })
      TweenMax.to(iconContainerView, duration, { scale: '.8', alpha: 0 })
      TweenMax.to(temperatureView, duration, { scale: '.8', alpha: 0 })
      TweenMax.to(lastUpdateView, duration, { y: '-20px', alpha: 0 })
      break
  }
}

function loadIcon (iconLoaded = null) {
  let currentIcon = document.createElement('object')
  currentIcon.id = `icon-${weatherData.lastUpdate}`
  currentIcon.className = 'icon'
  currentIcon.setAttribute('data', `static/icons/${weatherData.iconBase}.svg`)
  currentIcon.style.opacity = 0
  currentIcon.onload = () => {
    recolorIcon(currentIcon)
    if (iconLoaded) {
      if (typeof iconLoaded === 'function') {
        iconLoaded(currentIcon)
      } else {
        throw new Error('Icon loaded should be a function.')
      }
    }
  }
  iconContainerView.appendChild(currentIcon)
}

function updateIcon (animated = false , duration = 1) {
  loadIcon((icon) => {
    let prevIcons = []
    for (let i = 0; i < iconContainerView.children.length; i++) {
      if (iconContainerView.children[i] !== icon) {
        prevIcons.push(iconContainerView.children[i])
      }
    }
    if (!animated) {
      for (let i = 0; i < prevIcons.length; i++) {
        iconContainerView.removeChild(prevIcons[i])
      }
      icon.style.opacity = 1
    } else {
      TweenMax.to([prevIcons], 1, {
        alpha: 0,
        scale: 0.8,
        onComplete: () => {
          for (let i = 0; i < prevIcons.length; i++) {
            iconContainerView.removeChild(prevIcons[i])
          }
        }
      })
      TweenMax.to(icon, 1, { alpha: 1, delay: 0.5 })
    }
  })
}

function updateCity (animated = false , duration = 1) {
  let text = weatherData.cityName
  if (!animated) {
    cityView.innerHTML = text
  } else {
    TweenMax.to(cityView, duration / 2, {
      alpha: 0,
      onComplete: (view) => {
        cityView.innerHTML = text
        TweenMax.to(cityView, duration / 2, { alpha: 1 })
      }
    })
  }
}

function updateTemperature (animated = false , duration = 1) {
  let text = `${Math.round(weatherData.temperature)}°`
  if (!animated) {
    temperatureView.innerHTML = text
  } else {
    TweenMax.to(temperatureView, duration / 2, {
      alpha: 0,
      onComplete: (view) => {
        temperatureView.innerHTML = text
        TweenMax.to(temperatureView, duration / 2, { alpha: 1 })
      }
    })
  }
}

function updateLastUpdate (animated = false , duration = 1) {
  let d = new Date(weatherData.lastUpdate)
  let h = pad(d.getHours())
  let m = pad(d.getMinutes())
  let text = `Última actualización? Hoy ${h}:${m}hs`
  if (!animated) {
    lastUpdateView.innerHTML = text
  } else {
    TweenMax.to(lastUpdateView, duration / 2, {
      alpha: 0,
      onComplete: (view) => {
        lastUpdateView.innerHTML = text
        TweenMax.to(lastUpdateView, duration / 2, { alpha: 1 })
      }
    })
  }
}

function recolorIcon (svgObject) {
  var svgDoc = svgObject.contentDocument
  var defsElem = svgDoc.getElementsByTagName('defs')
  if (!defsElem || defsElem.length === 0) {
    defsElem = svgDoc.createElement('defs')
  }
  var styleElm = svgDoc.createElementNS('http://www.w3.org/1999/xhtml', 'style')
  styleElm.setAttribute('type', 'text/css')
  styleElm.innerHTML = require('!raw!./assets/css/icons.css')
  defsElem.appendChild(styleElm)
  svgDoc.getElementsByTagName('svg')[0].appendChild(defsElem)
}

function compareAttr (attr, objA, objB) {
  return (objA != null &&
  objB != null &&
  objA.hasOwnProperty(attr) &&
  objB.hasOwnProperty(attr) &&
  objA[attr] === objB[attr])
}

function pad (n) { return ('0' + n).slice(-2) }

ipcRenderer.on('start', (event, parameters) => {
  console.log('Module start received')
  console.log(parameters)
  weatherLib.init({
    uid: 334058856987399875,
    cityName: parameters.cityName,
    cityCode: parameters.cityCode,
    refreshInterval: parameters.refreshInterval * 1000
  })
})

ipcRenderer.on('update', (event, parameters) => {
  console.log('Module update received')
  weatherLib.update({
    cityName: parameters.cityName,
    cityCode: parameters.cityCode,
    refreshInterval: parameters.refreshInterval * 1000
  })
})

ipcRenderer.on('finish', (event) => {
  console.log('Module finish received')
  weatherLib.destroy()
  playAnimation('end')
})
